'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var web = require('solid-js/web');
var solidJs = require('solid-js');
var refs = require('@solid-primitives/refs');
var core = require('@solid-codemirror/core');
var view = require('@codemirror/view');

function createLineNumbers(props, createExtension) {
  const merged = solidJs.mergeProps({
    showLineNumbers: true
  }, props);
  const reconfigureLineNumbers = createExtension(getLineNumbers(merged.showLineNumbers));
  solidJs.createEffect(solidJs.on(() => merged.showLineNumbers, showLineNumbers => {
    reconfigureLineNumbers(getLineNumbers(showLineNumbers));
  }, {
    defer: true
  }));
}

function getLineNumbers(showLineNumbers) {
  return showLineNumbers ? view.lineNumbers() : [];
}

function createReadOnly(props, createExtension) {
  const merged = solidJs.mergeProps({
    readOnly: false
  }, props);
  const reconfigureLineNumbers = createExtension(getReadOnly(merged.readOnly));
  solidJs.createEffect(solidJs.on(() => merged.readOnly, readOnly => {
    reconfigureLineNumbers(getReadOnly(readOnly));
  }, {
    defer: true
  }));
}

function getReadOnly(readOnly) {
  return readOnly ? view.EditorView.editable.of(false) : [view.highlightActiveLine(), view.highlightActiveLineGutter()];
}

function createWrapLine(props, createExtension) {
  const merged = solidJs.mergeProps({
    wrapLine: false
  }, props);
  const reconfigureWrapLine = createExtension(getWrapLine(merged.wrapLine));
  solidJs.createEffect(solidJs.on(() => merged.wrapLine, wrapLine => {
    reconfigureWrapLine(getWrapLine(wrapLine));
  }, {
    defer: true
  }));
}

function getWrapLine(wrapLine) {
  return wrapLine ? view.EditorView.lineWrapping : [];
}

function createExtensions(props, createExtension) {
  const merged = solidJs.mergeProps({
    extensions: []
  }, props);
  const reconfigureExtensions = createExtension(merged.extensions);
  solidJs.createEffect(solidJs.on(() => merged.extensions, extensions => {
    reconfigureExtensions(extensions);
  }, {
    defer: true
  }));
}

function createTheme(props, createExtension) {
  const merged = solidJs.mergeProps({
    theme: []
  }, props);
  const reconfigureTheme = createExtension(merged.theme);
  solidJs.createEffect(solidJs.on(() => merged.theme, theme => {
    reconfigureTheme(theme);
  }, {
    defer: true
  }));
}

const _tmpl$ = /*#__PURE__*/web.template(`<div></div>`, 2);
function CodeMirror(props) {
  let ref;
  const [codemirrorProps, lineNumberProps, readOnlyProps, wrapLineProps, extensionProps, themeProps, others] = solidJs.splitProps(props, ["value", "onValueChange", "onEditorMount"], ["showLineNumbers"], ["readOnly"], ["wrapLine"], ["extensions"], ["theme"]);
  const {
    createExtension
  } = core.createCodeMirror(codemirrorProps, () => ref);
  createLineNumbers(lineNumberProps, createExtension);
  createReadOnly(readOnlyProps, createExtension);
  createWrapLine(wrapLineProps, createExtension);
  createTheme(themeProps, createExtension);
  createExtensions(extensionProps, createExtension);
  return (() => {
    const _el$ = _tmpl$.cloneNode(true);

    const _ref$ = refs.mergeRefs(el => ref = el, props.ref);

    typeof _ref$ === "function" && _ref$(_el$);

    web.spread(_el$, others, false, false);

    return _el$;
  })();
}

exports.CodeMirror = CodeMirror;
//# sourceMappingURL=index.common.js.map
