import { spread, template } from 'solid-js/web';
import { mergeProps, createEffect, on, splitProps } from 'solid-js';
import { mergeRefs } from '@solid-primitives/refs';
import { createCodeMirror } from '@solid-codemirror/core';
import { lineNumbers, EditorView, highlightActiveLine, highlightActiveLineGutter } from '@codemirror/view';

function createLineNumbers(props, createExtension) {
  const merged = mergeProps({
    showLineNumbers: true
  }, props);
  const reconfigureLineNumbers = createExtension(getLineNumbers(merged.showLineNumbers));
  createEffect(on(() => merged.showLineNumbers, showLineNumbers => {
    reconfigureLineNumbers(getLineNumbers(showLineNumbers));
  }, {
    defer: true
  }));
}

function getLineNumbers(showLineNumbers) {
  return showLineNumbers ? lineNumbers() : [];
}

function createReadOnly(props, createExtension) {
  const merged = mergeProps({
    readOnly: false
  }, props);
  const reconfigureLineNumbers = createExtension(getReadOnly(merged.readOnly));
  createEffect(on(() => merged.readOnly, readOnly => {
    reconfigureLineNumbers(getReadOnly(readOnly));
  }, {
    defer: true
  }));
}

function getReadOnly(readOnly) {
  return readOnly ? EditorView.editable.of(false) : [highlightActiveLine(), highlightActiveLineGutter()];
}

function createWrapLine(props, createExtension) {
  const merged = mergeProps({
    wrapLine: false
  }, props);
  const reconfigureWrapLine = createExtension(getWrapLine(merged.wrapLine));
  createEffect(on(() => merged.wrapLine, wrapLine => {
    reconfigureWrapLine(getWrapLine(wrapLine));
  }, {
    defer: true
  }));
}

function getWrapLine(wrapLine) {
  return wrapLine ? EditorView.lineWrapping : [];
}

function createExtensions(props, createExtension) {
  const merged = mergeProps({
    extensions: []
  }, props);
  const reconfigureExtensions = createExtension(merged.extensions);
  createEffect(on(() => merged.extensions, extensions => {
    reconfigureExtensions(extensions);
  }, {
    defer: true
  }));
}

function createTheme(props, createExtension) {
  const merged = mergeProps({
    theme: []
  }, props);
  const reconfigureTheme = createExtension(merged.theme);
  createEffect(on(() => merged.theme, theme => {
    reconfigureTheme(theme);
  }, {
    defer: true
  }));
}

const _tmpl$ = /*#__PURE__*/template(`<div></div>`, 2);
function CodeMirror(props) {
  let ref;
  const [codemirrorProps, lineNumberProps, readOnlyProps, wrapLineProps, extensionProps, themeProps, others] = splitProps(props, ["value", "onValueChange", "onEditorMount"], ["showLineNumbers"], ["readOnly"], ["wrapLine"], ["extensions"], ["theme"]);
  const {
    createExtension
  } = createCodeMirror(codemirrorProps, () => ref);
  createLineNumbers(lineNumberProps, createExtension);
  createReadOnly(readOnlyProps, createExtension);
  createWrapLine(wrapLineProps, createExtension);
  createTheme(themeProps, createExtension);
  createExtensions(extensionProps, createExtension);
  return (() => {
    const _el$ = _tmpl$.cloneNode(true);

    const _ref$ = mergeRefs(el => ref = el, props.ref);

    typeof _ref$ === "function" && _ref$(_el$);

    spread(_el$, others, false, false);

    return _el$;
  })();
}

export { CodeMirror };
//# sourceMappingURL=index.module.js.map
